==> 1.0.1-dev.tar.gz: lib/src/ui/draggable_bottom_sheet.dart <==
import 'package:flutter/material.dart';
import 'package:flutterx_application/flutterx_application.dart';
import 'package:flutterx_application/src/utils/no_overscroll_physics.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

/// Draggable bottom sheet based on [DraggableScrollableSheet]
///
/// This widget can be dragged along the vertical axis between its
/// [minChildSize], which defaults to `0.2` and [maxChildSize], which defaults
/// to `1.0`. The [initialChildSize] defaults to `0.6`
/// These sizes are percentages of the height of the parent container.
///
/// [minFocusableExtent] defines the minimum extent within the content can be focused, defaults to `0.5`.
/// This means that if you scroll down the BottomSheet lower that the 50% of the container height, the current focus is
/// cleared and the content will not be focusable until the extent returns over the threshold.
/// Setting this value to 0 means disabling focus check and content will be always focusable
///
/// If [updateInsets] is true, the bottom inset of [MediaQuery] will be updated as long as the extent changes.
/// This means that if you are using a [Scaffold] inside this BottomSheet with [Scaffold.resizeToAvoidBottomInset] true
/// it will automatically resized with a fluid animation. You may have to wrap that scaffold body with
/// a [SingleChildScrollView] with [NeverScrollableScrollPhysics] to avoid bottom overflow when scrolling down the
/// BottomSheet
///
/// [builder] allows you to build the content basing on [DraggableScrollableNotification]
class DraggableBottomSheet extends StatefulWidget {
  final double initialChildSize;
  final double minChildSize;
  final double maxChildSize;
  final double minFocusableExtent;
  final bool updateInsets;
  final Widget Function(BuildContext context, DraggableScrollableNotification notification) builder;

  const DraggableBottomSheet({
    Key? key,
    this.initialChildSize = .6,
    this.minChildSize = .2,
    this.maxChildSize = 1,
    this.minFocusableExtent = .5,
    this.updateInsets = true,
    required this.builder,
  }) : super(key: key);

  static ScrollController controllerOf(BuildContext context) =>
      context.findRootAncestorStateOfType<_DraggableBottomSheetState>()!._controller;

  @override
  State<DraggableBottomSheet> createState() => _DraggableBottomSheetState();
}

class _DraggableBottomSheetState extends State<DraggableBottomSheet> {
  late DraggableScrollableNotification _notification;
  late ScrollController _controller;

  @override
  void initState() {
    super.initState();
    _notification = DraggableScrollableNotification(
        extent: widget.initialChildSize,
        minExtent: widget.minChildSize,
        maxExtent: widget.maxChildSize,
        initialExtent: widget.initialChildSize,
        context: context);
  }

  @override
  Widget build(BuildContext context) {
    final originalMediaQuery = widget.updateInsets ? MediaQuery.of(context) : null;
    final focusScope = widget.minFocusableExtent > 0 ? FocusScope.of(context) : null;
    return NotificationListener<DraggableScrollableNotification>(
        onNotification: (notification) {
          _notification = notification;
          if (focusScope != null) {
            final focusable = notification.extent >= widget.minFocusableExtent;
            if (!focusable) focusScope.unfocus();
            focusScope.canRequestFocus = focusable;
          }
          return false;
        },
        child: DraggableScrollableSheet(
            minChildSize: widget.minChildSize,
            initialChildSize: widget.initialChildSize,
            maxChildSize: widget.maxChildSize,
            builder: (context, scrollController) {
              _controller = scrollController;
              final mediaQuery = originalMediaQuery?.let((mediaQuery) => mediaQuery.copyWith(
                  viewInsets: mediaQuery.viewInsets.copyWith(
                      bottom: (1 - _notification.extent) * mediaQuery.size.height + mediaQuery.viewInsets.bottom)));
              return SingleChildScrollView(
                  controller: scrollController,
                  physics: const NoOverScrollPhysics(),
                  child: wrapIfNotNull<Widget, MediaQueryData>(
                      item: mediaQuery,
                      wrap: (child, mediaQuery) => MediaQuery(data: mediaQuery, child: child),
                      child: widget.builder(context, _notification)));
            }));
  }
}

==> 1.0.9-dev.tar.gz: lib/src/ui/draggable_bottom_sheet.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutterx_application/flutterx_application.dart';
import 'package:flutterx_application/src/utils/no_overscroll_physics.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

/// Draggable bottom sheet based on [DraggableScrollableSheet]
///
/// This widget can be dragged along the vertical axis between its
/// [minChildSize], which defaults to `0.2` and [maxChildSize], which defaults
/// to `1.0`. The [initialChildSize] defaults to `0.6`
/// These sizes are percentages of the height of the parent container.
/// When running on web [initialChildSize] is equal to [maxChildSize] since drag is not supported
///
/// [minFocusableExtent] defines the minimum extent within the content can be focused, defaults to `0.5`.
/// This means that if you scroll down the BottomSheet lower that the 50% of the container height, the current focus is
/// cleared and the content will not be focusable until the extent returns over the threshold.
/// Setting this value to 0 means disabling focus check and content will be always focusable
///
/// If [updateInsets] is true, the bottom inset of [MediaQuery] will be updated as long as the extent changes.
/// This means that if you are using a [Scaffold] inside this BottomSheet with [Scaffold.resizeToAvoidBottomInset] true
/// it will automatically resized with a fluid animation. You may have to wrap that scaffold body with
/// a [SingleChildScrollView] with [NeverScrollableScrollPhysics] to avoid bottom overflow when scrolling down the
/// BottomSheet
/// When running on web this field is ignored since drag is not supported
///
/// [builder] allows you to build the content basing on [DraggableScrollableNotification]
class DraggableBottomSheet extends StatefulWidget {
  final double initialChildSize;
  final double minChildSize;
  final double maxChildSize;
  final double minFocusableExtent;
  final bool updateInsets;
  final Widget Function(BuildContext context, DraggableScrollableNotification notification) builder;

  const DraggableBottomSheet({
    Key? key,
    this.initialChildSize = .6,
    this.minChildSize = .2,
    this.maxChildSize = 1,
    this.minFocusableExtent = .5,
    this.updateInsets = true,
    required this.builder,
  }) : super(key: key);

  static ScrollController controllerOf(BuildContext context) =>
      context.findRootAncestorStateOfType<_DraggableBottomSheetState>()!._controller;

  @override
  State<DraggableBottomSheet> createState() => _DraggableBottomSheetState();
}

class _DraggableBottomSheetState extends State<DraggableBottomSheet> {
  late DraggableScrollableNotification _notification;
  late ScrollController _controller;

  @override
  void initState() {
    super.initState();
    _notification = DraggableScrollableNotification(
        extent: widget.initialChildSize,
        minExtent: widget.minChildSize,
        maxExtent: widget.maxChildSize,
        initialExtent: widget.initialChildSize,
        context: context);
  }

  @override
  Widget build(BuildContext context) {
    final originalMediaQuery = widget.updateInsets && !kIsWeb ? MediaQuery.of(context) : null;
    final focusScope = widget.minFocusableExtent > 0 ? FocusScope.of(context) : null;
    return NotificationListener<DraggableScrollableNotification>(
        onNotification: (notification) {
          _notification = notification;
          if (focusScope != null) {
            final focusable = notification.extent >= widget.minFocusableExtent;
            if (!focusable) focusScope.unfocus();
            focusScope.canRequestFocus = focusable;
          }
          return false;
        },
        child: DraggableScrollableSheet(
            minChildSize: widget.minChildSize,
            initialChildSize: kIsWeb ? widget.maxChildSize : widget.initialChildSize,
            maxChildSize: widget.maxChildSize,
            builder: (context, scrollController) {
              _controller = scrollController;
              final mediaQuery = originalMediaQuery?.let((mediaQuery) => mediaQuery.copyWith(
                  viewInsets: mediaQuery.viewInsets.copyWith(
                      bottom: (1 - _notification.extent) * mediaQuery.size.height + mediaQuery.viewInsets.bottom)));
              return SingleChildScrollView(
                  controller: scrollController,
                  physics: const NoOverScrollPhysics(),
                  child: wrapIfNotNull<Widget, MediaQueryData>(
                      item: mediaQuery,
                      wrap: (child, mediaQuery) => MediaQuery(data: mediaQuery, child: child),
                      child: widget.builder(context, _notification)));
            }));
  }
}
